
/*
 * Copyright (c) Developers of JPolynomial
 * Redistributable and modifiable. Please see license.readme for details.
 *
 * ammendment history
 * <date>			<changed by>		<summary of changes>
 * 27 August 2004	Jonee Ryan Ty		First documented history
 *
 */

public class Monomial {
	private double coef = 1;
	private int pow = 1;
	
	/**
	 *parses a monomial string
	 */
	 
	public Monomial(String monoStr)		throws IllegalArgumentException {
		boolean isSigned = 
			(Math.max(monoStr.indexOf('+'), monoStr.indexOf('-')) != -1);
		
		int indexOfX = Math.max(monoStr.indexOf('x'), monoStr.indexOf('X')),
			indexOfKaret	= monoStr.indexOf('^');

		if ((indexOfX != -1) && (indexOfX != 0)) {		
			// 'x' or 'X' exists in monoStr and it does not start the String 	
			if (!(isSigned && (indexOfX == 1))) 		
				// 'x' or 'X' does not follow a sign char	
				coef = Double.parseDouble(monoStr.substring(0, indexOfX));
													
			else if (monoStr.indexOf('-') != -1)		
				// if 'x' or 'X' follows '-' coef is -1 
				coef = -1;								
		}
		
		if (indexOfKaret != -1) {		// the karet exists in monoStr

			if (indexOfX == -1)
				throw new IllegalArgumentException("karet without X");
	
			else {
				try {
					pow = Integer.parseInt(monoStr.substring(indexOfKaret +1));
				
				} catch (NumberFormatException nfe) {	
					throw new NumberFormatException("number format exception");
				}
			}
		}
		
		if ((indexOfX == -1) && (indexOfKaret == -1)) { // input is number only		
			coef = Double.parseDouble(monoStr);
			pow = 0;
		}				

	} // end of Monomial(String)
	
	public double getCoef() {
		return coef;
	}
		
	public int getPow() {
		return pow;
	}
	
	public void setCoef(double coef) {
		this.coef = coef;
	}
	
	public void setPow(int pow) {
		this.pow = pow;
	}
	
	/**
	 * returns a String representation of this monomial with a leading sign
	 */
	public String toString() {
		String monoStr = new String();
		Monomial tmpMono = this;
				/*to process power first*/
		if ((tmpMono.getPow() != 1) && (tmpMono.getPow() != 0))
			monoStr = "x^" + Integer.toString(tmpMono.getPow());
			
		else if (tmpMono.getPow() == 1)
			monoStr += "x";		
		// else if pow == 0 then do nothing
		
				/*process coefficent afterwards*/
		if (tmpMono.getCoef() > 0) {
			if (tmpMono.getCoef() != 1)	
				monoStr = " + " + Double.toString(tmpMono.getCoef()) +
					monoStr;
					
			else {
				if (tmpMono.getPow() == 0)
					monoStr = " + 1";
						
				else	
					monoStr = " + " + monoStr;
			}
			
		} else if (tmpMono.getCoef() == 0)
			monoStr = " + 0";
				
		else {
			if (tmpMono.getCoef() != -1)	
				monoStr = " - " + Double.toString(-tmpMono.getCoef()) +
					monoStr;
				
			else {
				if (tmpMono.getPow() == 0)
					monoStr = " - 1"; 
					
				else
					monoStr = " - " + monoStr;
			}
		}
		
		return monoStr;		
	} // toString
	
	public String toString(boolean removeLeadingPlus) {
		String 	ret = this.toString(),
				tmp;
		
		if (removeLeadingPlus) {
			tmp = ret.trim();
			
			if (tmp.charAt(0) == '+')
				ret = tmp.substring(2);	
		} 
		
		return ret;
	}
	
} //##30
