
/*
 * Copyright (c) Developers of JPolynomial
 * Redistributable and modifiable. Please see license.readme for details.
 *
 * ammendment history
 * <date>			<changed by>		<summary of changes>
 * 27 August 2004	Jonee Ryan Ty		First documented history
 *
 */

//import java.awt.Graphics;
import java.awt.Color;
import java.awt.Font;
import java.awt.Cursor;
//import java.awt.Image;

import java.awt.event.MouseEvent;
import javax.swing.JTextArea;

/**
 *the drawing board
 */

public class PolyCanv		extends javax.swing.JPanel
							implements 	java.awt.event.MouseMotionListener,
										java.awt.event.MouseListener
{                    
	public static final int 	NONE	= 0,		// axis types
								FRAMED	= 2,
								NORMAL	= 1,
		
								TOP_MARGIN = 22,
								LEFT_MARGIN = 22,
								RIGHT_MARGIN = 22,
								  //number of intervals the x - axis is divided
								X_PARTITION = 8, //program can accomodate
								 		// only even value for this one
								VER_STRIP_LEN = 4,  // length of a strip in the
														// x - axis
								HOR_STRIP_LEN = 4,
								Y_PARTITION = 8,

								BOTTOM_MARGIN = VER_STRIP_LEN / 2 +	35;
				// 10 + 17 for heights of labelFont, othersFont, 8 allowance
					// canvas coordinates where the mouse is pointing at
	public int		typeOfAxis = 1,

					graphWidth = 256,
					graphHeight = 256,

					sweep;

	public boolean 	showLabels = true,// if axes labels are shown
					isTraversing = false;//if the graph traversal is being done

	public double	highX = 5,	//axis extreme values
					lowX = -5,
					highY = 10,
					lowY = -10;

	public Function fToGraph;
	
	public JTextArea whereToPrint;
	
	private int p[] = new int[2];

	private static boolean 	mouseMoved = false,
							mouseMovedInside = false,// inside graphing bounds
							printMsg = false;
							
	private String msg;							
								// font for axes values
	private Font 	labelFont = new Font("Arial", Font.PLAIN, 8),
					othersFont = new Font("TimesRoman", Font.PLAIN, 12);
		
	public PolyCanv() {
		super();
		this.setPreferredSize(new java.awt.Dimension(
			graphWidth + LEFT_MARGIN + RIGHT_MARGIN,
            graphHeight + TOP_MARGIN + BOTTOM_MARGIN));
		this.setCursor(new Cursor(Cursor.CROSSHAIR_CURSOR));
		this.setBackground(Color.white);
		this.addMouseMotionListener(this);
		this.addMouseListener(this);
	}

	public void mouseClicked(MouseEvent e) {}
	public void mouseDragged(MouseEvent e) {}
	public void mouseEntered(MouseEvent e) {}

	public void mouseExited(MouseEvent e) {
		if(fToGraph == null || isTraversing)
			return;
			//when mouse exited clear the display for coordinates
		mouseMoved = true;
								// 17 height of othersFont + 6 allowance total
		repaint(0, TOP_MARGIN + graphHeight + BOTTOM_MARGIN - 24,
			this.getWidth(), 24);
	}

	public void mousePressed(MouseEvent e) {}
	public void mouseReleased(MouseEvent e) {}

	public void mouseMoved(MouseEvent e) {
		if(fToGraph == null || isTraversing)
			return;
							//whenever mouse moves inside graphing bounds,
		mouseMoved = true;		// the coordinates are displayed

		p[0] = e.getX();
		p[1] = e.getY();

			// if mouse moved inside graph boundaries
		if((p[0] >= LEFT_MARGIN) && (p[0] <= LEFT_MARGIN + graphWidth) &&
			(p[1] >= TOP_MARGIN) && (p[1] <= TOP_MARGIN + graphHeight))
			mouseMovedInside = true;

			//when mouse moves outside, clear the coordinates display
		repaint(0, TOP_MARGIN + graphHeight + BOTTOM_MARGIN - 24,
			this.getWidth(), 24);
	}

	public void paintComponent(java.awt.Graphics g) {

		super.paintComponent(g);

		if(fToGraph == null) {
			g.drawString(Constants.COPYRIGHT, 10, 15);
			return;
		}

		double 	height = highY - lowY,
				width = highX - lowX,

				xIntervals = width / X_PARTITION,	// strip intervals
				yIntervals = height / Y_PARTITION,
							// the length in pixels of intervals
				xSkip =	(double)graphWidth / X_PARTITION,
				ySkip = (double)graphHeight / Y_PARTITION,

				xNow = lowX + xIntervals, // values represented by strips
				yNow = lowY + yIntervals,

				xProp	= graphWidth / width,
				yProp 	= graphHeight / height,
				inc 	= width / graphWidth, // increments in x per pixel
													// in graphing
				x1, x2;			// drawLine variables
			// canvas coordinates counterpart to x1, x2
		long 	cx1, cx2, cy1, cy2;

		String sTmp;	//used for writting labels

			/*to draw axis and label if necessary*/
		g.setColor(Color.BLACK);
		g.setFont(labelFont);

		typeOfAxis %= 3;
		
		if(typeOfAxis == NORMAL) {
			g.drawLine(LEFT_MARGIN + graphWidth / 2 , TOP_MARGIN,
				LEFT_MARGIN + graphWidth / 2, TOP_MARGIN + graphHeight);

			g.drawLine(LEFT_MARGIN, TOP_MARGIN + graphHeight / 2,
				LEFT_MARGIN + graphWidth, TOP_MARGIN + graphHeight / 2);

			for(int a = 1; a < X_PARTITION; a++) {
					//to draw vertical strips from left to right
				g.drawLine(LEFT_MARGIN + (int)(a * xSkip),
					TOP_MARGIN + (graphHeight - VER_STRIP_LEN) / 2,
					LEFT_MARGIN + (int)(a * xSkip),
					TOP_MARGIN + (graphHeight + VER_STRIP_LEN) / 2);
			}

			for(int b = 1; b < Y_PARTITION; b++) {
				//to draw horizontal strips from bottom to top
				g.drawLine(LEFT_MARGIN + (graphWidth - HOR_STRIP_LEN) / 2,
					TOP_MARGIN + graphHeight - (int)(b * (ySkip)),
					LEFT_MARGIN + (graphWidth + HOR_STRIP_LEN) / 2,
					TOP_MARGIN + graphHeight - (int)(b * ySkip));
			}

			if(showLabels) {		
				// to draw x-axis labels skipping the middle
				for(int c = 1; c < X_PARTITION / 2; c++) {
					sTmp = Double.toString(xNow);
						// x-axis label
					g.drawString(sTmp,
						LEFT_MARGIN +
						(int)(c * (xSkip)) - (sTmp.length() * 5) / 2,
						TOP_MARGIN + 1 + (graphHeight + VER_STRIP_LEN) / 2 +
						g.getFontMetrics().getHeight());
					// - (sTmp.length() * 5) / 2 allowance in x, 1 pixel in y
					// 5 pixels is estimated to be the thickness of a letter
					// using point 8
					xNow += xIntervals;
				}

				xNow += xIntervals;

				for(int d = (X_PARTITION + 2) / 2; d < X_PARTITION; d++) {
					sTmp = Double.toString(xNow);

					g.drawString(sTmp,
						LEFT_MARGIN +
						(int)(d * (xSkip)) - (sTmp.length() * 5) / 2,
						TOP_MARGIN + 1 + (graphHeight + VER_STRIP_LEN) / 2 +
						g.getFontMetrics().getHeight());

					xNow += xIntervals;
				}

				for(int e = 1; e < Y_PARTITION / 2; e++) {
					g.drawString(Double.toString(yNow),
						LEFT_MARGIN + 4 + (graphWidth + HOR_STRIP_LEN) / 2,
						TOP_MARGIN + graphHeight +
						g.getFontMetrics().getHeight() / 2 - (int)(e * ySkip));
							// 4 pixels allowance in x,
								// g.getFontMetrics().getHeight() / 2 in y
					yNow += yIntervals;
				}

				yNow += yIntervals;

				for(int f = (Y_PARTITION + 2) / 2; f < Y_PARTITION; f++) {
					g.drawString(Double.toString(yNow),
						LEFT_MARGIN + 4 + (graphWidth + HOR_STRIP_LEN) / 2,
						TOP_MARGIN + graphHeight +
						g.getFontMetrics().getHeight() / 2 - (int)(f * ySkip));

					yNow += yIntervals;
				}
			} // if(showLabel)
			// if(typeOfAxis == 1)
		} else if(typeOfAxis == FRAMED) {
			g.drawLine(LEFT_MARGIN, TOP_MARGIN, LEFT_MARGIN,
				TOP_MARGIN + graphHeight);

			g.drawLine(LEFT_MARGIN, TOP_MARGIN + graphHeight,
				LEFT_MARGIN + graphWidth, TOP_MARGIN + graphHeight);

			for(int h = 1; h < X_PARTITION; h++) {
				g.drawLine(LEFT_MARGIN + (int)(h * xSkip),
					TOP_MARGIN + graphHeight - VER_STRIP_LEN / 2,
					LEFT_MARGIN + (int)(h * xSkip),
					TOP_MARGIN + graphHeight + VER_STRIP_LEN / 2);
			}

			for(int i = 1; i < Y_PARTITION; i++) {
				g.drawLine(LEFT_MARGIN - HOR_STRIP_LEN / 2,
					TOP_MARGIN + graphHeight - (int)(i * ySkip),
					LEFT_MARGIN + HOR_STRIP_LEN / 2,
					TOP_MARGIN + graphHeight - (int)(i * ySkip));
			}

			if(showLabels) {
				for(int j = 1; j < X_PARTITION; j++) {
					sTmp = Double.toString(xNow);

					g.drawString(sTmp,
						LEFT_MARGIN +
						(int)(j * (xSkip)) - (sTmp.length() * 5) / 2,
						TOP_MARGIN + graphHeight + 1 + VER_STRIP_LEN / 2 +
						g.getFontMetrics().getHeight());

					xNow += xIntervals;
				}

				for(int k = 1; k < Y_PARTITION; k++) {
					g.drawString(Double.toString(yNow),
						LEFT_MARGIN + 4 + HOR_STRIP_LEN / 2,
						TOP_MARGIN + graphHeight +
						g.getFontMetrics().getHeight() / 2 - (int)(k * ySkip));

					yNow += yIntervals;
				}
			}
			// if(typeOfAxis == 2)
		
		}
		
		if (printMsg) {
			g.drawString(msg, 10, TOP_MARGIN + graphHeight + BOTTOM_MARGIN - 5);
			printMsg = false;
		}					
					/*to draw coordinate display if necessary*/
		if(mouseMoved) {
			mouseMoved = false;

			if(mouseMovedInside) {
				g.setColor(Color.MAGENTA);
				g.setFont(othersFont);

				sTmp = "p(" + Double.toString(lowX +
					(p[0] - LEFT_MARGIN) / xProp) + ", " + Double.toString(
					(TOP_MARGIN + graphHeight - p[1]) / yProp + lowY) + ")";

				g.drawString(sTmp, LEFT_MARGIN + graphWidth -
					(int)(5.5 * sTmp.length()),
					TOP_MARGIN + graphHeight + BOTTOM_MARGIN - 5);
		// 5 allowance, 5.5 estimated thickness of a letter using othersFont
				mouseMovedInside = false;
			
			} else {
				//g.clearRect(0, TOP_MARGIN + graphHeight + BOTTOM_MARGIN - 23,
				//	this.getWidth(), 23);
						// 17 height of othersFont + 6 allowance total
			}
		}
			/*to draw function*/
		g.setColor(Color.BLUE);

		x1 = lowX;

		for(int l = 0; l < graphWidth; l++) {
			x2 = x1 + inc;

			cx1 = LEFT_MARGIN + (long)(xProp * (x1 - lowX));
			cx2 = LEFT_MARGIN + (long)(xProp * (x2 - lowX));
			cy1 = TOP_MARGIN + graphHeight -
					(long)(yProp * (fToGraph.f(x1) - lowY));
			cy2 = TOP_MARGIN + graphHeight -
					(long)(yProp * (fToGraph.f(x2) - lowY));

			x1 = x2; // increment x1

				//draw only in the graphing region
			if((cy1 > TOP_MARGIN) && (cy1 < TOP_MARGIN + graphHeight) &&
				(cy2 > TOP_MARGIN) && (cy2 < TOP_MARGIN + graphHeight))
				g.drawLine((int)cx1, (int)cy1, (int)cx2, (int)cy2);

					// if cy2 is in the region and cy1 at top margin
			else if((cy1 < TOP_MARGIN) &&
				(cy2 > TOP_MARGIN) && (cy2 < TOP_MARGIN + graphHeight))
				g.drawLine((int)((TOP_MARGIN - cy1) * (cx1 - cx2) /
					(double)(cy1 - cy2) + cx1), TOP_MARGIN, (int)cx2, (int)cy2);

					//if cy1 is in the region and cy2 at top margin
			else if((cy1 > TOP_MARGIN) && (cy1 < TOP_MARGIN + graphHeight) &&
				(cy2 < TOP_MARGIN))
				g.drawLine((int)cx1, (int)cy1,
					(int)((TOP_MARGIN - cy1) * (cx1 - cx2) / (double)(cy1 - cy2)
					+ cx1), TOP_MARGIN);

					//if cy1 is in the region and cy2 at bottom margin
			else if((cy1 > TOP_MARGIN) && (cy1 < TOP_MARGIN + graphHeight) &&
				(cy2 > TOP_MARGIN + graphHeight))
				g.drawLine((int)cx1, (int)cy1,
					(int)((TOP_MARGIN + graphHeight - cy1) * (cx1 - cx2) /
						(double)(cy1 - cy2) + cx1), TOP_MARGIN + graphHeight);

					// if cy2 is in the region and cy1 at bottom margin
			else if((cy1 > TOP_MARGIN + graphHeight) &&
				(cy2 > TOP_MARGIN) && (cy2 < TOP_MARGIN + graphHeight))
				g.drawLine((int)((TOP_MARGIN + graphHeight - cy1) *
					(cx1 - cx2) / (double)(cy1 - cy2) + cx1),
					TOP_MARGIN + graphHeight, (int)cx2, (int)cy2);
		}
			/* to write the function at the top margin*/
		g.setFont(othersFont);
		g.setColor(Color.GREEN);

		g.drawString("f(x) = " + fToGraph.toString(), LEFT_MARGIN,
			TOP_MARGIN - 6);	// 6 = allowance

			/*if traversing the graph*/
		if(isTraversing) {
			g.setColor(Color.RED);
			x1 = lowX + (sweep - LEFT_MARGIN) / xProp;	//re-used variables
			x2 = lowX + (sweep - LEFT_MARGIN + 1) / xProp;

			double 	y1 = fToGraph.f(x1),
					y2 = fToGraph.f(x2);

			cy1 = TOP_MARGIN + graphHeight - (long)(yProp * (y1 - lowY));
			cy2 = TOP_MARGIN + graphHeight - (long)(yProp * (y2 - lowY));

			if((cy1 > TOP_MARGIN) && (cy1 < TOP_MARGIN + graphHeight))
				g.fillOval(sweep - 3, (int)cy1 - 3, 6, 6);

			if(y1 == 0) {
				PolyApp.print(x1 + " is a root in this interval", 
					whereToPrint);
			}
				// the IVT
			else if(((y1 < 0) && (y2 > 0)) || ((y1 > 0) && (y2 < 0))) {
				PolyApp.print("A root can be found between " + x1 +
					" and " + x2, whereToPrint);
			}
		}
	} // end of paint()

	public void printMsg(String msg) {
		printMsg = true;
		this.msg = msg;

		repaint(0, TOP_MARGIN + graphHeight + BOTTOM_MARGIN - 55, 
			this.getWidth(), 50);
	}
	
}//##30
