
/*
 * Copyright (c) Developers of JPolynomial
 * Redistributable and modifiable. Please see license.readme for details.
 *
 * ammendment history
 * <date>			<changed by>		<summary of changes>
 * 27 August 2004	Jonee Ryan Ty		First documented history
 *
 */
 
import java.util.StringTokenizer;
import java.util.Vector;

//import javax.swing.JTextField;	//reference to textfields where to write 
						// the nested forms of f(x) and fp(x) where needed
																		
/**
 * problem: can't accept signed exponents because '-+' is a delimiter
 * and anyway, this is a polynomial class
 */

public class Polynomial		extends Function
{
	private Vector v;	//container for monomials
	
	private int k;	//universal counter for methods f() and fp() to save memory
						// because these functions will be used very frequently
	public double coefs[];	// this will be used in evaluation using
								//horner's method
	/**
	 *parses a polynomial string representation such as "x^2 + 2x + 8"
	 */	 
	public Polynomial(String polyStr)	throws IllegalArgumentException {
		if (polyStr.length() == 0)
			throw new IllegalArgumentException("empty string");
		
		StringTokenizer toks = new StringTokenizer(polyStr.trim());
		
		String 	s1 = new String(), //temp string objects
				s2 = new String();
		
		Monomial tmpMono;
		
		int 	a, 						
				tmpPow;
		
		boolean placeFound = false;		//used when adding a monomial to v
		
		v = new Vector(5, 1);
		
		polyStr = new String();	// to re-use variable		
				/*to remove white spaces*/
		while (toks.hasMoreTokens())	{
			polyStr += toks.nextToken();
		}
				//if at this point polyStr does not start with a sign, 
		if (!(polyStr.charAt(0) == '+' || polyStr.charAt(0) == '-'))
			polyStr = "+" + polyStr; // then attach one
		
				// to tokenize polyStr
		toks = new StringTokenizer(polyStr, "+-", true); 
				/*to process the tokens*/
		while (toks.hasMoreTokens())	{
			s1 = toks.nextToken();
			
			if (s1.equals("+") || s1.equals("-")) {// if token is a sign then
													// attach it to next token
				if (s2.equals("+") || s2.equals("-"))
					throw new IllegalArgumentException("double sign");
				
				else s2 = s1;	// store it to s2 for now
			
			} else {	//if token is not a sign, the string s2 + s1 
						//represents a monomial
				try {			//exception is anticipated here
					tmpMono = new Monomial(s2 + s1);	
					s2 = new String();
				
				} catch (IllegalArgumentException err) {
					// number format exception will also be caught here
					throw err;
				}
			
				if (tmpMono.getCoef() != 0) {//only monomials with non-zero 
					// coefficients are added to container, those with zero 
					placeFound = false;		//coefficients are ignored
						/*to add the monomial to its place in container v*/
					// the monomials are arranged according to ascending power
					for (a = 0; a < v.size(); a++) {
						tmpPow = ((Monomial)v.elementAt(a)).getPow();		 
									
						if (tmpMono.getPow() == tmpPow) {	
							// to combine two monomials with equal powers
							tmpMono.setCoef(tmpMono.getCoef() 
								+ ((Monomial)v.remove(a)).getCoef());
							
							if (tmpMono.getCoef() != 0)
								v.insertElementAt(tmpMono, a);
								
							a = v.size(); // to end loop
							placeFound = true;	
						
						} else if (tmpMono.getPow() < tmpPow) {	
							v.insertElementAt(tmpMono, a);
							a = v.size(); // to end loop
							placeFound = true;
						}
					}
						//to insert monomial at the end,
					if (!placeFound)	//this happens when monomial has highest 
						v.addElement(tmpMono); // power at the instance that it
								// is inserted or container is still empty
				}
			}
		} // end of while
	
		System.gc();
			/* to get coefficients for horner's method later*/
		if (v.size() == 0)
			coefs[0] = 0;
			
		else {			
			coefs = new double[((Monomial)v.lastElement()).getPow() + 1];
			
			for (int b = 0; b < v.size(); b++) {	
			//in reverse order, coefficient of least power at last of array
				tmpMono = (Monomial)v.elementAt(b);
				coefs[coefs.length - tmpMono.getPow() - 1] = tmpMono.getCoef();
			}		//this is in synthetic division order
		}
	} // end of Polynomial()
	
	/**
	 * evaluates polynomial at x using horner's method,
	 * coefficients stored in reverse order, coefficient of least 
	 * power at last of array
	 */	 
	public double f(double x) {
		double px = coefs[0];
		
		for (k = 1; k < coefs.length; k++)
			px = coefs[k] + px * x;
			
		return px;
	}
	
	/**
	 * evaluates derivative of polynomial at x using horner's method,
	 * coefficients still in reverse
	 */	
	public double fp(double x) {
		double dp = (coefs.length - 1) * coefs[0];
		
		for (k = 1; k < coefs.length - 1; k++)
			dp = (coefs.length - 1 - k) * coefs[k] + dp * x;
		
		return dp;		
	}
	
	/**
	 * makes a string representation of the polynomial object
	 */	 
	public String toString() {
		String polStr = new String();				
			//make a string representation of each monomial then concatenate
		for (int c = 0; c < v.size(); c++)
			polStr += ((Monomial)v.elementAt(c)).toString();
		
		if (v.size() == 0)
			polStr = "0";	// the zero polynomial if container is empty
		
		polStr = polStr.trim();
				
		if (polStr.charAt(0) == '+') //to remove a leading '+' in polynomial 
			polStr = polStr.substring(2); // strings such as "+ 3x - 4"
		
		return polStr;
	}
	
	/**
	 * expresses the derivative this polynomial to its nested form 
	 */	 
	public String fpxToNestedForm() {
		Polynomial p = this;
		
		String 	s1	= new String(),
				s2	= new String();	//closing parentheses part
			
		if (p.coefs.length == 1) {
			return "0";
		}
				// note that coefficients are in reversed order
		s1 += Double.toString(p.coefs[p.coefs.length - 2]);
		
		if (p.coefs.length == 2)			
			return s1;	
		
		for (int e = p.coefs.length - 3; e > 0; e--) {
			s1 += " + x(" + Double.toString((p.coefs.length - e - 1) * 
				p.coefs[e]);
			s2 += ")";	
		} 

		if (p.coefs[0] > 0) {
			if (p.coefs[0] * (p.coefs.length - 1) == 1)
				s1 += " + x";
				
			else
				s1 += " + " + 
					Double.toString((p.coefs.length - 1)* p.coefs[0]) + "x";
		
		} else {	
			if (p.coefs[0] * (p.coefs.length - 1) == -1)
				s1 += " - x";
				
			else
				s1 += " - " + 
					Double.toString((-p.coefs.length + 1)* p.coefs[0]) + "x";
		}
		
		return s1 + s2;
	}
	
	/**
	 *expresses this polynomial to its nested form 
	 */	 
	public String fxToNestedForm() {
		Polynomial p = this;
		String 	s1	= new String(),
				s2	= new String();	//closing parentheses part
			// note that coefficients are in reversed order
		s1 += Double.toString(p.coefs[p.coefs.length - 1]);
		
		if (p.coefs.length == 1)	// constant polynomial
			return s1;
		
		for (int e = p.coefs.length - 2; e > 0; e--) {
			s1 += " + x(" + Double.toString(p.coefs[e]);
			s2 += ")";	
		} 
				
		if (p.coefs[0] > 0) {
			if(p.coefs[0] == 1)
				s1 += " + x";
				
			else
				s1 += " + " + Double.toString(p.coefs[0]) + "x";
		
		
		} else {	
			if (p.coefs[0] == -1)
				s1 += " - x";
				
			else
				s1 += " - " + Double.toString(-p.coefs[0]) + "x";
		}
		
		return s1 + s2;
	}

}//##30
